﻿
#include "Funktionen.h"
#include "main.h"


ISR ( PWM_OVERFLOW )
{
	static uint8_t Startup = 100;
	static uint8_t Count = 0;
	static uint8_t MuxCount = 0;
	static uint16_t Debounce = 0;
	static uint8_t OldMux = MUX_U;

	TIFR1 |= (1<<OCF1B);
	
	//////////////////////////////////////////////////////////////////////////
	Count++;
	Values.RPMCount++;
	
	// Read ADC and start next conversation
	switch ( OldMux ){
		case MUX_U:				Values.VoltageRaw = ADC; break;
		case MUX_I:				Values.CurrentRaw = ADC; break;
		case MUX_FLUXTHRESHOLD: Values.FluxThreshold = ADC; break;
		case MUX_BEMF1:			Values.Temp= ADC; break;
		case MUX_BEMF2:			Values.Temp= ADC; break;
		case MUX_BEMF3:			Values.Temp= ADC; break;
		default: break; 
	}
	OldMux = ADMUX;
	
	// Wait until ADC is sure started before preparing ADMUX for next measurement!!!
	do {
		asm volatile ("NOP");
	}while (!(ADCSRA & (1<<ADSC)));

	// Set MUX
	if ( Count %2 == 0 ){
		switch ( CommutationVector ){
			case 1: ADMUX = MUX_BEMF2; break;
			case 2: ADMUX = MUX_BEMF1; break;
			case 3: ADMUX = MUX_BEMF3; break;
			case 4: ADMUX = MUX_BEMF2; break;
			case 5: ADMUX = MUX_BEMF1; break;
			case 6: ADMUX = MUX_BEMF3; break;
			default: break;
		}
	}
	else{
		if ( ++MuxCount >= 3 ) MuxCount = 0;
		ADMUX = MuxCount;
	}
	
	//////////////////////////////////////////////////////////////////////////
	// Shut down Powerstage if Overcurrent, else drive on
	if ( Values.CurrentRaw > OVERCURRENTTHRESHOLD || Values.State == 0 ){
		POWERSTAGE_OFF();
	}else if ( Values.State == 1 ){
		RestartCommutation();
		Values.State = 2;
	}
	
	//////////////////////////////////////////////////////////////////////////
	if ( Debounce ) Debounce--;
	
	//////////////////////////////////////////////////////////////////////////
	// Wait until all Measurements are sure done
	if ( Startup ){
		Startup--;
	}else if ( Count %2 == 1 && Values.State ) { // Only integrate Flux when Flux was measured!!!
		// Start Integrating if Voltage is above/below Ub/2
		if ( CommutationVector % 2 == 1 ){
			if ( Values.Temp > (Values.VoltageRaw/2) ){
				Values.Fluxintegral += (Values.Temp - (Values.VoltageRaw/2));
			}
		}else{
			if ( Values.Temp < (Values.VoltageRaw/2) ){
				Values.Fluxintegral += ((Values.VoltageRaw/2) -Values.Temp);
			}
		}
		// Integrationthreshold reached?
		if ( (Values.Fluxintegral >= Values.FluxThreshold) ){
			if ( ++CommutationVector >= 7 ) {
				CommutationVector = 1;
				Values.RPM = (10000ul * 30) / (Values.RPMCount) ;
				Values.RPMCount = 1;
			}
			if ( Values.State ) NextCommutation();
			Values.Fluxintegral = 0;
		}
	}else if (Values.State == 0){
		Values.Fluxintegral = 0;
	}
	
	//////////////////////////////////////////////////////////////////////////
	// Scan Startbutton
	if ( ((!(PINB & (1<<PINB0))) && (!Debounce)) ){
		if (Values.State) Values.State = 0;
		else Values.State = 1;
		Debounce = DEBOUNCETIME;
	}
	
	//////////////////////////////////////////////////////////////////////////
	// Force Commutation if Rotor is in wrong postion
	if ( Values.ForceCommutate ){
		Values.ForceCommutate--;
	}else {
		if ( Values.State ) {
			if ( ++CommutationVector >= 7 ) CommutationVector = 1;
			Values.ForceCommutate = FORCE_COMMUTATION_TIME;
		}
	}
	
	//////////////////////////////////////////////////////////////////////////
	// Speed up Engine
	if ( Values.State ){
		if (Count % 1024 == 0 ){
			if (OCR1A < 600){
		//		OCR1A++;
			}
		}
	}else{
		OCR1A = STARTDUTY;
	}
	
	//////////////////////////////////////////////////////////////////////////
	
}

